local L = LibStub("AceLocale-3.0"):GetLocale("TrackOMatic", true);

--==================================================
-- Initialize the main menu
--==================================================
function TrackOMatic_InitMainMenu()
    CreateFrame("Frame", "TrackOMatic_TrackerMenu", UIParent, "UIDropDownMenuTemplate");
end

--==================================================
-- Handler for clicking the track button
-- (magnifying glass); Displays the main menu
--==================================================
function TrackOMatic_TrackButton_OnClick(self)
    local menu = {
        { text = L["ADDON_TITLE"], isTitle = true, notCheckable = true },
        { text = "Add Tracker", hasArrow = true, notCheckable = true,
            menuList = {
                { text = L["MENU_TRACK_REPUTATION"], notCheckable = true, func = function() ToggleCharacter("ReputationFrame"); end },
                { text = L["MENU_TRACK_PROFESSION"], notCheckable = true, func = function() ToggleSpellBook(BOOKTYPE_PROFESSION); end },
                { text = L["MENU_TRACK_CURRENCY"], notCheckable = true, func = function() ToggleCharacter("TokenFrame"); end },
                { text = L["MENU_TRACK_ACHIEVEMENT"], notCheckable = true, func = function() ToggleAchievementFrame(); end },
                { text = L["MENU_TRACK_ITEM"], notCheckable = true, func = function() CloseDropDownMenus(); StaticPopup_Show("TRACKOMATIC_TRACK_ITEM"); end },
                { text = L["MENU_TRACK_PLAYER"], hasArrow = true, notCheckable = true, 
                    menuList = {
                        { text = L["PLAYER_INFO_XP"], disabled = TrackOMatic_IsTracked("player", "xp"), notCheckable = true, func = function() TrackOMatic_AddPlayerInfo("xp"); end },
                        { text = L["PLAYER_INFO_KILLS_TO_LEVEL"], disabled = TrackOMatic_IsTracked("player", "grind"), notCheckable = true, func = function() TrackOMatic_AddPlayerInfo("grind"); end },
                        { text = L["PLAYER_INFO_QUESTS_TO_LEVEL"], disabled = TrackOMatic_IsTracked("player", "quest"), notCheckable = true, func = function() TrackOMatic_AddPlayerInfo("quest"); end },
                        { text = L["PLAYER_INFO_GOLD"], disabled = TrackOMatic_IsTracked("player", "gold"), notCheckable = true, func = function() TrackOMatic_AddPlayerInfo("gold"); end },
                        { text = L["PLAYER_INFO_DURABILITY"], disabled = TrackOMatic_IsTracked("player", "durability"), notCheckable = true, func = function() TrackOMatic_AddPlayerInfo("durability"); end },
                        { text = L["PLAYER_INFO_HONOR"], disabled = TrackOMatic_IsTracked("player", "honor"), notCheckable = true, func = function() TrackOMatic_AddPlayerInfo("honor"); end },
                    },
                },
                { text = L["MENU_TRACK_EVENTTRIGGER"], hasArrow = true, notCheckable = true,
                    menuList = {
                        { text = L["EVENT_TRIGGER_0001"], disabled = TrackOMatic_IsTracked("eventtrigger", "0001"), notCheckable = true, func = function() TrackOMatic_AddEventTrigger("0001"); end },
                        { text = L["EVENT_TRIGGER_0002"], disabled = TrackOMatic_IsTracked("eventtrigger", "0002"), notCheckable = true, func = function() TrackOMatic_AddEventTrigger("0002"); end },
                    },
                },
                -- to-do: add misc category once there's something to go into it
                { text = L["MENU_TRACK_PLUGIN"], hasArrow = true, notCheckable = true,
                    menuList = TrackOMatic_GetMenuPluginList(),
                },
            },
        },
        { text = L["MENU_CLEAR_CATEGORY"], hasArrow = true, notCheckable = true,
            menuList = {
                { text = L["MENU_CLEAR_ALL"], notCheckable = true, func = function() TrackOMatic_ClearTracker(); end },
                { text = "", notClickable = true, notCheckable = true },
                { text = L["HEADER_REPUTATION"], notCheckable = true, disabled = TrackOMatic_CategoryIsEmpty("rep"), func = function() TrackOMatic_ClearCategory("rep"); end },
                { text = L["HEADER_PROFESSIONS"], notCheckable = true, disabled = TrackOMatic_CategoryIsEmpty("skill"), func = function() TrackOMatic_ClearCategory("skill"); end },
                { text = L["HEADER_PLAYER"], notCheckable = true, disabled = TrackOMatic_CategoryIsEmpty("player"), func = function() TrackOMatic_ClearCategory("player"); end },
                { text = L["HEADER_CURRENCY"], notCheckable = true, disabled = TrackOMatic_CategoryIsEmpty("currency"), func = function() TrackOMatic_ClearCategory("currency"); end },
                { text = L["HEADER_ITEMS"], notCheckable = true, disabled = TrackOMatic_CategoryIsEmpty("item"), func = function() TrackOMatic_ClearCategory("item"); end },
                { text = L["HEADER_ACHIEVEMENTS"], notCheckable = true, disabled = TrackOMatic_CategoryIsEmpty("ach"), func = function() TrackOMatic_ClearCategory("ach"); end },
                { text = L["HEADER_GUILD_ACHIEVEMENTS"], notCheckable = true, disabled = TrackOMatic_CategoryIsEmpty("guild_ach"), func = function() TrackOMatic_ClearCategory("guild_ach"); end },
                { text = L["HEADER_EVENT_TRIGGERS"], notCheckable = true, disabled = TrackOMatic_CategoryIsEmpty("eventtrigger"), func = function() TrackOMatic_ClearCategory("eventtrigger"); end },
                { text = L["HEADER_MISC"], notCheckable = true, disabled = TrackOMatic_CategoryIsEmpty("misc"), func = function() TrackOMatic_ClearCategory("misc"); end },
                { text = L["HEADER_PLUGINS"], notCheckable = true, disabled = TrackOMatic_CategoryIsEmpty("plugin"), func = function() TrackOMatic_ClearCategory("plugin"); end },
            },
        },
        { text = L["MENU_TEMPLATES"], hasArrow = true, notCheckable = true,
            menuList = TrackOMatic_GetMenuTemplateList(),
        },
        { text = L["MENU_UPDATE_INTERVAL"], hasArrow = true, notCheckable = true,
            menuList = {
                { text = L["MENU_UPDATE_INTERVAL_1"], checked = (TRACKOMATIC_VARS['config']['update_interval'] == 0), func = function() TrackOMatic_SetUpdateInterval(0); end },
                { text = L["MENU_UPDATE_INTERVAL_2"], checked = (TRACKOMATIC_VARS['config']['update_interval'] == 1), func = function() TrackOMatic_SetUpdateInterval(1); end },
                { text = L["MENU_UPDATE_INTERVAL_3"], checked = (TRACKOMATIC_VARS['config']['update_interval'] == 2), func = function() TrackOMatic_SetUpdateInterval(2); end },
                { text = L["MENU_UPDATE_INTERVAL_4"], checked = (TRACKOMATIC_VARS['config']['update_interval'] == 3), func = function() TrackOMatic_SetUpdateInterval(3); end },
                { text = L["MENU_UPDATE_INTERVAL_5"], checked = (TRACKOMATIC_VARS['config']['update_interval'] == 4), func = function() TrackOMatic_SetUpdateInterval(4); end },
            },
        },
        { text = L["MENU_SHOW_MOUSEOVER"], checked = TrackOMatic.AutoHide, func = function() TrackOMatic_SetAutoHide(not TrackOMatic.AutoHide); end },
        { text = "", notClickable = true, notCheckable = true },
        { text = L["MENU_CONFIGURE"], notCheckable = true, func = function() InterfaceOptionsFrame_OpenToCategory(TrackOMaticConfigContainer); end },
    };

    TrackOMatic_RunHooks("MAIN_MENU", menu);

    EasyMenu(menu, TrackOMatic_TrackerMenu, self, 0, 0, "MENU", 1);
end
